@@ -6,13 +6,20 @@ use warnings;
 use utf8;
 use Mouse;
 use Mouse::Util::TypeConstraints;
+use Locale::TextDomain qw(App-Sqitch);
+use App::Sqitch::X qw(hurl);
 use List::Util qw(first);
 use namespace::autoclean;
 extends 'App::Sqitch::Command';
 
-our $VERSION = '0.983';
+our $VERSION = '0.990';
 
-has to_target => (
+has target => (
+    is  => 'ro',
+    isa => 'Str',
+);
+
+has to_change => (
     is  => 'ro',
     isa => 'Str',
 );
@@ -53,11 +60,13 @@ has variables => (
 
 sub options {
     return qw(
-        to-target|to|target=s
+        target|t=s
+        to-change|to|change=s
         mode=s
         set|s=s%
         log-only
         verify!
+        to-target=s
     );
 }
 
@@ -69,7 +78,16 @@ sub configure {
         verify   => $opt->{verify} // $config->get( key => 'deploy.verify', as => 'boolean' ) // 0,
         log_only => $opt->{log_only} || 0,
     );
-    $params{to_target} = $opt->{to_target} if exists $opt->{to_target};
+    $params{to_change} = $opt->{to_change} if exists $opt->{to_change};
+    $params{target}    = $opt->{target}    if exists $opt->{target};
+
+    if ( exists $opt->{to_target} ) {
+        # Deprecated option.
+        App::Sqitch->warn(
+            __ 'The --to-target and --target option has been deprecated; use --to-change instead.'
+        );
+        $params{to_change} ||= $opt->{to_target};
+    }
 
     if ( my $vars = $opt->{set} ) {
         # Merge with config.
@@ -83,12 +101,39 @@ sub configure {
 }
 
 sub execute {
-    my $self   = shift;
-    my $engine = $self->sqitch->engine;
+    my $self = shift;
+    my %args = $self->parse_args(@_);
+
+    # Die on unknowns.
+    if (my @unknown = @{ $args{unknown}} ) {
+        hurl deploy => __nx(
+            'Unknown argument "{arg}"',
+            'Unknown arguments: {arg}',
+            scalar @unknown,
+            arg => join ', ', @unknown
+        );
+    }
+
+    # Warn on multiple targets.
+    my $target = $self->target // shift @{ $args{targets} };
+    $self->warn(__x(
+        'Too many targets specified; connecting to {target}',
+        target => $target,
+    )) if @{ $args{targets} };
+
+    # Warn on too many changes.
+    my $change = $self->to_change // shift @{ $args{changes} };
+    $self->warn(__x(
+        'Too many changes specified; deploying to "{change}"',
+        change => $change,
+    )) if @{ $args{changes} };
+
+    # Now get to work.
+    my $engine = $self->engine_for_target($target);
     $engine->with_verify( $self->verify );
     $engine->log_only( $self->log_only );
     if (my %v = %{ $self->variables }) { $engine->set_variables(%v) }
-    $engine->deploy( $self->to_target // shift, $self->mode );
+    $engine->deploy( $change, $self->mode );
     return $self;
 }
 
