@@ -14,7 +14,12 @@ use Try::Tiny;
 use namespace::autoclean;
 extends 'App::Sqitch::Command';
 
-our $VERSION = '0.983';
+our $VERSION = '0.990';
+
+has target => (
+    is  => 'ro',
+    isa => 'Str',
+);
 
 has show_changes => (
     is      => 'ro',
@@ -56,7 +61,7 @@ has project => (
     default => sub {
         my $self = shift;
         try { $self->plan->project } || do {
-            # Try to extract a project name from the database.
+            # Try to extract a project name from the registry.
             my $engine = $self->engine;
             hurl status => __ 'Database not initialized for Sqitch'
                 unless $engine->initialized;
@@ -74,6 +79,7 @@ has project => (
 sub options {
     return qw(
         project=s
+        target|t=s
         show-tags
         show-changes
         date-format|date=s
@@ -81,8 +87,15 @@ sub options {
 }
 
 sub execute {
-    my $self   = shift;
-    my $engine = $self->engine;
+    my ( $self, $target ) = @_;
+
+    # Warn on multiple targets.
+    $self->warn(__x(
+        'Both the --target option and the target argument passed; using {option}',
+        option => $self->target,
+    )) if $target && $self->target;
+
+    my $engine = $self->engine_for_target($self->target // $target);
 
     # Where are we?
     $self->comment( __x 'On database {db}', db => $engine->destination );
@@ -93,7 +106,10 @@ sub execute {
     } catch {
         # Hrm. Maybe not initialized?
         die $_ if $engine->initialized;
-        hurl status => __ 'Database not initialized for Sqitch';
+        hurl status => __x(
+            'Database {db} has not been initialized for Sqitch',
+            db => $engine->registry_destination
+        );
     };
 
     hurl {
@@ -300,8 +316,8 @@ works, read on.
 
   $status->execute;
 
-Executes the status command. The current state of the database will be compared
-to the plan in order to show where things stand.
+Executes the status command. The current state of the target database will be
+compared to the plan in order to show where things stand.
 
 =head3 C<emit_changes>
 
@@ -319,15 +335,15 @@ Emits a list of deployed tags if C<show_tags> is true.
 
   $status->emit_state($state);
 
-Emits the current state of the database. Pass in a state hash as returned by
-L<App::Sqitch::Engine> C<current_state()>.
+Emits the current state of the target database. Pass in a state hash as
+returned by L<App::Sqitch::Engine> C<current_state()>.
 
 =head3 C<emit_status>
 
   $status->emit_state($state);
 
-Emits information about the current status of the database compared to the
-plan. Pass in a state hash as returned by L<App::Sqitch::Engine>
+Emits information about the current status of the target database compared to
+the plan. Pass in a state hash as returned by L<App::Sqitch::Engine>
 C<current_state()>. Throws an exception if the current state's change cannot
 be found in the plan.
 
