@@ -6,18 +6,25 @@ use warnings;
 use utf8;
 use Mouse;
 use Mouse::Util::TypeConstraints;
+use App::Sqitch::X qw(hurl);
+use Locale::TextDomain qw(App-Sqitch);
 use List::Util qw(first);
 use namespace::autoclean;
 extends 'App::Sqitch::Command';
 
-our $VERSION = '0.983';
+our $VERSION = '0.990';
 
-has from_target => (
+has target => (
     is  => 'ro',
     isa => 'Str',
 );
 
-has to_target => (
+has from_change => (
+    is  => 'ro',
+    isa => 'Str',
+);
+
+has to_change => (
     is  => 'ro',
     isa => 'Str',
 );
@@ -34,8 +41,11 @@ has variables => (
 
 sub options {
     return qw(
-        from-target|from=s
-        to-target|to=s
+        target|t=s
+        from-change|from=s
+        to-change|to=s
+        from-target=s
+        to-target=s
         set|s=s%
     );
 }
@@ -47,7 +57,19 @@ sub configure {
         $_ => $opt->{$_}
     } grep {
         exists $opt->{$_}
-    } qw(from_target to_target);
+    } qw(target from_change to_change);
+
+    # Handle deprecated options.
+    for my $key (qw(from to)) {
+        if (my $val = $opt->{"$key\_target"}) {
+            App::Sqitch->warn(__x(
+                'Option --{old} has been deprecated; use --{new} instead',
+                old => "$key-target",
+                new => "$key-change",
+            ));
+            $params{"$key\_change"} ||= $val;
+        }
+    }
 
     if ( my $vars = $opt->{set} ) {
         # Merge with config.
@@ -61,10 +83,39 @@ sub configure {
 }
 
 sub execute {
-    my $self   = shift;
-    my $engine = $self->sqitch->engine;
+    my $self = shift;
+    my %args = $self->parse_args(@_);
+
+    # Die on unknowns.
+    if (my @unknown = @{ $args{unknown}} ) {
+        hurl verify => __nx(
+            'Unknown argument "{arg}"',
+            'Unknown arguments: {arg}',
+            scalar @unknown,
+            arg => join ', ', @unknown
+        );
+    }
+
+    # Warn on multiple targets.
+    my $target = $self->target // shift @{ $args{targets} };
+    $self->warn(__x(
+        'Too many targets specified; connecting to {target}',
+        target => $target,
+    )) if @{ $args{targets} };
+
+    # Warn on too many changes.
+    my $from = $self->from_change // shift @{ $args{changes} };
+    my $to   = $self->to_change   // shift @{ $args{changes} };
+    $self->warn(__x(
+        'Too many changes specified; verifying from "{from}" to "{to}"',
+        from => $from,
+        to   => $to,
+    )) if @{ $args{changes} };
+
+    # Now get to work.
+    my $engine = $self->engine_for_target($target);
     if (my %v = %{ $self->variables }) { $engine->set_variables(%v) }
-    $engine->verify( $self->from_target // shift, $self->to_target // shift );
+    $engine->verify($from, $to);
     return $self;
 }
 
