@@ -18,7 +18,7 @@ use namespace::autoclean;
 use Mouse;
 use constant SYNTAX_VERSION => '1.0.0-b2';
 
-our $VERSION = '0.983';
+our $VERSION = '0.990';
 
 # Like [:punct:], but excluding _. Copied from perlrecharclass.
 my $punct = q{-!"#$%&'()*+,./:;<=>?@[\\]^`{|}~};
@@ -571,7 +571,8 @@ sub check_changes {
 
 sub open_script {
     my ( $self, $file ) = @_;
-    return $file->open('<:utf8_strict') or hurl plan => __x(
+    # return has higher precedence than or, so use ||.
+    return $file->open('<:utf8_strict') || hurl plan => __x(
         'Cannot open {file}: {error}',
         file  => $file,
         error => $!,
@@ -585,6 +586,7 @@ sub tags           { shift->_changes->tags }
 sub count          { shift->_changes->count }
 sub index_of       { shift->_changes->index_of(shift) }
 sub get            { shift->_changes->get(shift) }
+sub contains       { shift->_changes->contains( shift ) }
 sub find           { shift->_changes->find(shift) }
 sub first_index_of { shift->_changes->first_index_of(@_) }
 sub change_at      { shift->_changes->change_at(shift) }
@@ -1099,6 +1101,13 @@ The named change as it was last seen in the list before the specified tag.
 
 =back
 
+=head3 C<contains>
+
+  say 'Yes!' if $plan->contains('6c2f28d125aff1deea615f8de774599acf39a7a1');
+
+Like C<index_of()>, but never throws an exception, and returns true if the
+plan contains the specified change, and false if it does not.
+
 =head3 C<get>
 
   my $change = $plan->get('6c2f28d125aff1deea615f8de774599acf39a7a1');
